@@ -5,13 +5,10 @@
 #include <inttypes.h>
 #include <libgen.h>
 #include <stdio.h>
-#include <SDL2/SDL_thread.h>
 #include <SDL2/SDL_timer.h>
 #include <SDL2/SDL_platform.h>
 
-#include "config.h"
-#include "command.h"
-#include "util/lock.h"
+#include "adb.h"
 #include "util/log.h"
 #include "util/net.h"
 #include "util/str_util.h"
@@ -34,7 +31,7 @@ get_server_path(void) {
 #ifdef __WINDOWS__
         char *server_path = utf8_from_wide_char(server_path_env);
 #else
-        char *server_path = SDL_strdup(server_path_env);
+        char *server_path = strdup(server_path_env);
 #endif
         if (!server_path) {
             LOGE("Could not allocate memory");
@@ -46,7 +43,7 @@ get_server_path(void) {
 
 #ifndef PORTABLE
     LOGD("Using server: " DEFAULT_SERVER_PATH);
-    char *server_path = SDL_strdup(DEFAULT_SERVER_PATH);
+    char *server_path = strdup(DEFAULT_SERVER_PATH);
     if (!server_path) {
         LOGE("Could not allocate memory");
         return NULL;
@@ -61,19 +58,32 @@ get_server_path(void) {
         LOGE("Could not get executable path, "
              "using " SERVER_FILENAME " from current directory");
         // not found, use current directory
-        return SERVER_FILENAME;
+        return strdup(SERVER_FILENAME);
     }
-    char *dir = dirname(executable_path);
+
+    // dirname() does not work correctly everywhere, so get the parent
+    // directory manually.
+    // See <https://github.com/Genymobile/scrcpy/issues/2619>
+    char *p = strrchr(executable_path, PATH_SEPARATOR);
+    if (!p) {
+        LOGE("Unexpected executable path: \"%s\" (it should contain a '%c')",
+             executable_path, PATH_SEPARATOR);
+        free(executable_path);
+        return strdup(SERVER_FILENAME);
+    }
+
+    *p = '\0'; // modify executable_path in place
+    char *dir = executable_path;
     size_t dirlen = strlen(dir);
 
     // sizeof(SERVER_FILENAME) gives statically the size including the null byte
     size_t len = dirlen + 1 + sizeof(SERVER_FILENAME);
-    char *server_path = SDL_malloc(len);
+    char *server_path = malloc(len);
     if (!server_path) {
         LOGE("Could not alloc server path string, "
              "using " SERVER_FILENAME " from current directory");
-        SDL_free(executable_path);
-        return SERVER_FILENAME;
+        free(executable_path);
+        return strdup(SERVER_FILENAME);
     }
 
     memcpy(server_path, dir, dirlen);
@@ -81,7 +91,7 @@ get_server_path(void) {
     memcpy(&server_path[dirlen + 1], SERVER_FILENAME, sizeof(SERVER_FILENAME));
     // the final null byte has been copied with SERVER_FILENAME
 
-    SDL_free(executable_path);
+    free(executable_path);
 
     LOGD("Using server (portable): %s", server_path);
     return server_path;
@@ -96,36 +106,36 @@ push_server(const char *serial) {
     }
     if (!is_regular_file(server_path)) {
         LOGE("'%s' does not exist or is not a regular file\n", server_path);
-        SDL_free(server_path);
+        free(server_path);
         return false;
     }
     process_t process = adb_push(serial, server_path, DEVICE_SERVER_PATH);
-    SDL_free(server_path);
-    return process_check_success(process, "adb push");
+    free(server_path);
+    return process_check_success(process, "adb push", true);
 }
 
 static bool
 enable_tunnel_reverse(const char *serial, uint16_t local_port) {
     process_t process = adb_reverse(serial, SOCKET_NAME, local_port);
-    return process_check_success(process, "adb reverse");
+    return process_check_success(process, "adb reverse", true);
 }
 
 static bool
 disable_tunnel_reverse(const char *serial) {
     process_t process = adb_reverse_remove(serial, SOCKET_NAME);
-    return process_check_success(process, "adb reverse --remove");
+    return process_check_success(process, "adb reverse --remove", true);
 }
 
 static bool
 enable_tunnel_forward(const char *serial, uint16_t local_port) {
     process_t process = adb_forward(serial, local_port, SOCKET_NAME);
-    return process_check_success(process, "adb forward");
+    return process_check_success(process, "adb forward", true);
 }
 
 static bool
 disable_tunnel_forward(const char *serial, uint16_t local_port) {
     process_t process = adb_forward_remove(serial, local_port);
-    return process_check_success(process, "adb forward --remove");
+    return process_check_success(process, "adb forward --remove", true);
 }
 
 static bool
@@ -238,6 +248,8 @@ enable_tunnel_any_port(struct server *server, struct sc_port_range port_range,
 static const char *
 log_level_to_server_string(enum sc_log_level level) {
     switch (level) {
+        case SC_LOG_LEVEL_VERBOSE:
+            return "verbose";
         case SC_LOG_LEVEL_DEBUG:
             return "debug";
         case SC_LOG_LEVEL_INFO:
@@ -258,12 +270,13 @@ execute_server(struct server *server, const struct server_params *params) {
     char bit_rate_string[11];
     char max_fps_string[6];
     char lock_video_orientation_string[5];
-    char display_id_string[6];
+    char display_id_string[11];
     sprintf(max_size_string, "%"PRIu16, params->max_size);
     sprintf(bit_rate_string, "%"PRIu32, params->bit_rate);
     sprintf(max_fps_string, "%"PRIu16, params->max_fps);
-    sprintf(lock_video_orientation_string, "%"PRIi8, params->lock_video_orientation);
-    sprintf(display_id_string, "%"PRIu16, params->display_id);
+    sprintf(lock_video_orientation_string, "%"PRIi8,
+            params->lock_video_orientation);
+    sprintf(display_id_string, "%"PRIu32, params->display_id);
     const char *const cmd[] = {
         "shell",
         "CLASSPATH=" DEVICE_SERVER_PATH,
@@ -272,7 +285,8 @@ execute_server(struct server *server, const struct server_params *params) {
 # define SERVER_DEBUGGER_PORT "5005"
 # ifdef SERVER_DEBUGGER_METHOD_NEW
         /* Android 9 and above */
-        "-XjdwpProvider:internal -XjdwpOptions:transport=dt_socket,suspend=y,server=y,address="
+        "-XjdwpProvider:internal -XjdwpOptions:transport=dt_socket,suspend=y,"
+        "server=y,address="
 # else
         /* Android 8 and below */
         "-agentlib:jdwp=transport=dt_socket,suspend=y,server=y,address="
@@ -296,6 +310,7 @@ execute_server(struct server *server, const struct server_params *params) {
         params->stay_awake ? "true" : "false",
         params->codec_options ? params->codec_options : "-",
         params->encoder_name ? params->encoder_name : "-",
+        params->power_off_on_close ? "true" : "false",
     };
 #ifdef SERVER_DEBUGGER
     LOGI("Server debugger waiting for a client on device port "
@@ -308,7 +323,7 @@ execute_server(struct server *server, const struct server_params *params) {
     //     Port: 5005
     // Then click on "Debug"
 #endif
-    return adb_execute(server->serial, cmd, sizeof(cmd) / sizeof(cmd[0]));
+    return adb_execute(server->serial, cmd, ARRAY_LEN(cmd));
 }
 
 static socket_t
@@ -358,18 +373,17 @@ bool
 server_init(struct server *server) {
     server->serial = NULL;
     server->process = PROCESS_NONE;
-    server->wait_server_thread = NULL;
     atomic_flag_clear_explicit(&server->server_socket_closed,
                                memory_order_relaxed);
 
-    server->mutex = SDL_CreateMutex();
-    if (!server->mutex) {
+    bool ok = sc_mutex_init(&server->mutex);
+    if (!ok) {
         return false;
     }
 
-    server->process_terminated_cond = SDL_CreateCond();
-    if (!server->process_terminated_cond) {
-        SDL_DestroyMutex(server->mutex);
+    ok = sc_cond_init(&server->process_terminated_cond);
+    if (!ok) {
+        sc_mutex_destroy(&server->mutex);
         return false;
     }
 
@@ -379,8 +393,6 @@ server_init(struct server *server) {
     server->video_socket = INVALID_SOCKET;
     server->control_socket = INVALID_SOCKET;
 
-    server->port_range.first = 0;
-    server->port_range.last = 0;
     server->local_port = 0;
 
     server->tunnel_enabled = false;
@@ -392,12 +404,12 @@ server_init(struct server *server) {
 static int
 run_wait_server(void *data) {
     struct server *server = data;
-    cmd_simple_wait(server->process, NULL); // ignore exit code
+    process_wait(server->process, false); // ignore exit code
 
-    mutex_lock(server->mutex);
+    sc_mutex_lock(&server->mutex);
     server->process_terminated = true;
-    cond_signal(server->process_terminated_cond);
-    mutex_unlock(server->mutex);
+    sc_cond_signal(&server->process_terminated_cond);
+    sc_mutex_unlock(&server->mutex);
 
     // no need for synchronization, server_socket is initialized before this
     // thread was created
@@ -412,30 +424,28 @@ run_wait_server(void *data) {
 }
 
 bool
-server_start(struct server *server, const char *serial,
-             const struct server_params *params) {
-    server->port_range = params->port_range;
-
-    if (serial) {
-        server->serial = SDL_strdup(serial);
+server_start(struct server *server, const struct server_params *params) {
+    if (params->serial) {
+        server->serial = strdup(params->serial);
         if (!server->serial) {
             return false;
         }
     }
 
-    if (!push_server(serial)) {
-        goto error1;
+    if (!push_server(params->serial)) {
+        /* server->serial will be freed on server_destroy() */
+        return false;
     }
 
     if (!enable_tunnel_any_port(server, params->port_range,
                                 params->force_adb_forward)) {
-        goto error1;
+        return false;
     }
 
     // server will connect to our server socket
     server->process = execute_server(server, params);
     if (server->process == PROCESS_NONE) {
-        goto error2;
+        goto error;
     }
 
     // If the server process dies before connecting to the server socket, then
@@ -444,19 +454,19 @@ server_start(struct server *server, const char *serial,
     // things simple and multiplatform, just spawn a new thread waiting for the
     // server process and calling shutdown()/close() on the server socket if
     // necessary to wake up any accept() blocking call.
-    server->wait_server_thread =
-        SDL_CreateThread(run_wait_server, "wait-server", server);
-    if (!server->wait_server_thread) {
-        cmd_terminate(server->process);
-        cmd_simple_wait(server->process, NULL); // ignore exit code
-        goto error2;
+    bool ok = sc_thread_create(&server->wait_server_thread, run_wait_server,
+                               "wait-server", server);
+    if (!ok) {
+        process_terminate(server->process);
+        process_wait(server->process, true); // ignore exit code
+        goto error;
     }
 
     server->tunnel_enabled = true;
 
     return true;
 
-error2:
+error:
     if (!server->tunnel_forward) {
         bool was_closed =
             atomic_flag_test_and_set(&server->server_socket_closed);
@@ -466,13 +476,32 @@ error2:
         close_socket(server->server_socket);
     }
     disable_tunnel(server);
-error1:
-    SDL_free(server->serial);
+
     return false;
 }
 
+static bool
+device_read_info(socket_t device_socket, char *device_name, struct size *size) {
+    unsigned char buf[DEVICE_NAME_FIELD_LENGTH + 4];
+    ssize_t r = net_recv_all(device_socket, buf, sizeof(buf));
+    if (r < DEVICE_NAME_FIELD_LENGTH + 4) {
+        LOGE("Could not retrieve device information");
+        return false;
+    }
+    // in case the client sends garbage
+    buf[DEVICE_NAME_FIELD_LENGTH - 1] = '\0';
+    // strcpy is safe here, since name contains at least
+    // DEVICE_NAME_FIELD_LENGTH bytes and strlen(buf) < DEVICE_NAME_FIELD_LENGTH
+    strcpy(device_name, (char *) buf);
+    size->width = (buf[DEVICE_NAME_FIELD_LENGTH] << 8)
+            | buf[DEVICE_NAME_FIELD_LENGTH + 1];
+    size->height = (buf[DEVICE_NAME_FIELD_LENGTH + 2] << 8)
+            | buf[DEVICE_NAME_FIELD_LENGTH + 3];
+    return true;
+}
+
 bool
-server_connect_to(struct server *server) {
+server_connect_to(struct server *server, char *device_name, struct size *size) {
     if (!server->tunnel_forward) {
         server->video_socket = net_accept(server->server_socket);
         if (server->video_socket == INVALID_SOCKET) {
@@ -512,7 +541,8 @@ server_connect_to(struct server *server) {
     disable_tunnel(server); // ignore failure
     server->tunnel_enabled = false;
 
-    return true;
+    // The sockets will be closed on stop if device_read_info() fails
+    return device_read_info(server->video_socket, device_name, size);
 }
 
 void
@@ -536,33 +566,33 @@ server_stop(struct server *server) {
     }
 
     // Give some delay for the server to terminate properly
-    mutex_lock(server->mutex);
-    int r = 0;
+    sc_mutex_lock(&server->mutex);
+    bool signaled = false;
     if (!server->process_terminated) {
-#define WATCHDOG_DELAY_MS 1000
-        r = cond_wait_timeout(server->process_terminated_cond,
-                              server->mutex,
-                              WATCHDOG_DELAY_MS);
+#define WATCHDOG_DELAY SC_TICK_FROM_SEC(1)
+        signaled = sc_cond_timedwait(&server->process_terminated_cond,
+                                     &server->mutex,
+                                     sc_tick_now() + WATCHDOG_DELAY);
     }
-    mutex_unlock(server->mutex);
+    sc_mutex_unlock(&server->mutex);
 
     // After this delay, kill the server if it's not dead already.
     // On some devices, closing the sockets is not sufficient to wake up the
     // blocking calls while the device is asleep.
-    if (r == SDL_MUTEX_TIMEDOUT) {
-        // FIXME There is a race condition here: there is a small chance that
-        // the process is already terminated, and the PID assigned to a new
-        // process.
+    if (!signaled) {
+        // The process is terminated, but not reaped (closed) yet, so its PID
+        // is still valid.
         LOGW("Killing the server...");
-        cmd_terminate(server->process);
+        process_terminate(server->process);
     }
 
-    SDL_WaitThread(server->wait_server_thread, NULL);
+    sc_thread_join(&server->wait_server_thread, NULL);
+    process_close(server->process);
 }
 
 void
 server_destroy(struct server *server) {
-    SDL_free(server->serial);
-    SDL_DestroyCond(server->process_terminated_cond);
-    SDL_DestroyMutex(server->mutex);
+    free(server->serial);
+    sc_cond_destroy(&server->process_terminated_cond);
+    sc_mutex_destroy(&server->mutex);
 }
