@@ -1,6 +1,8 @@
 #ifndef NET_H
 #define NET_H
 
+#include "common.h"
+
 #include <stdbool.h>
 #include <stdint.h>
 #include <SDL2/SDL_platform.h>
@@ -17,8 +19,6 @@
   typedef int socket_t;
 #endif
 
-#include "config.h"
-
 bool
 net_init(void);
 
