@@ -1,8 +1,9 @@
+#include "common.h"
+
 #include <assert.h>
 #include <string.h>
 
 #include "cli.h"
-#include "common.h"
 #include "scrcpy.h"
 
 static void test_flag_version(void) {
@@ -50,14 +51,13 @@ static void test_options(void) {
         "--fullscreen",
         "--max-fps", "30",
         "--max-size", "1024",
-        "--lock-video-orientation", "2",
+        "--lock-video-orientation=2", // optional arguments require '='
         // "--no-control" is not compatible with "--turn-screen-off"
         // "--no-display" is not compatible with "--fulscreen"
         "--port", "1234:1236",
         "--push-target", "/sdcard/Movies",
         "--record", "file",
         "--record-format", "mkv",
-        "--render-expired-frames",
         "--serial", "0123456789abcdef",
         "--show-touches",
         "--turn-screen-off",
@@ -86,7 +86,6 @@ static void test_options(void) {
     assert(!strcmp(opts->push_target, "/sdcard/Movies"));
     assert(!strcmp(opts->record_filename, "file"));
     assert(opts->record_format == SC_RECORD_FORMAT_MKV);
-    assert(opts->render_expired_frames);
     assert(!strcmp(opts->serial, "0123456789abcdef"));
     assert(opts->show_touches);
     assert(opts->turn_screen_off);
